/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 * PendEvent.java,v 1.2 2002/08/02 15:39:21 jba Exp
 */

package jca.util;

import jca.Ca;
import java.lang.Thread;


/*
 * This class gives the user the possibility to turn on and off a thread that will pend events periodically.
 * The period and time during which the thread will pend the events are configurable.
 * This is very useful when using "only" asynchronous functions because you don't have to worry about pending
 * events.
 *
 * @see jca.Ca.pendEvent
 */
public final class PendEvent implements Runnable {
  /* 
   *This is the default period (in second).
   */
  public static final double DEFAULT_PERIOD= 99E-3;
  /* 
   *This is the default pend time (in second).
   */
  public static final double DEFAULT_TIME  =  1E-6;
  /*
   * Start the PendEvent thread.
   */
  public static void start() {
    if(m_pend==null) m_pend= new PendEvent();
  }

  /*
   * Stop the PendEvent thread.
   */
  public static void stop() {
    if(m_pend!=null) m_pend.exit();
    m_pend= null;
  }

  /*
   * Set the PendEvent period.
   *
   * @param period  the period in second. It will be rounded to the nearest millisecond.
   */
  public static void setPeriod(double period) {
    m_period= (long) (1000*period);
  }

  /*
   * Get the PendEvent period.
   */
  public static double getPeriod() {
    return 0.001*m_period;
  }

  /*
   * Set the PendEvent pend time.
   *
   * @param time  the pend time in second.
   */
  public static void setTime(double time) {
    m_time= time;
  }

  /*
   * Get the PendEvent pend time.
   */
  public static double getTime() {
    return m_time;
  }

  /*
   * The unique instance of PendEvent.
   */
  private static PendEvent m_pend= null;

  /*
   * The PendEvent period.
   */
  private static long        m_period= (long) (1000*DEFAULT_PERIOD);

  /*
   * The PendEvent pend time.
   */
  private static double      m_time= DEFAULT_TIME;




  /*
   * Private ctor.
   */
  private PendEvent() {
    m_exit= false;
    m_thread= new Thread(this);
    m_thread.setPriority(Thread.NORM_PRIORITY-1);
    m_thread.start();
  }

  /*
   * This function tells the thread to exit the main loop.
   */
  private void exit() {
    m_exit= true;
  }

  /*
   * The Thread main loop.
   */
  public void run() {
    try {
      while(!m_exit) {
        Ca.pendEvent(m_time);
        m_thread.sleep(m_period);
      }
    } catch(InterruptedException e) {
    } finally {
    }
  }

  private Thread  m_thread= null;
  private boolean m_exit  = false;
}
